package net.rumati.logging.muffero;

/**
 * A simple utility class for retrieving a logger by name. All methods in this
 * class call the non-static method of the same name on the current
 * {@link LoggerContext}.
 * @author Jesse Long
 * @see LoggerContext
 */
public class LoggerFactory
{
    /**
     * Private constructor. This is a utility class. No construction required.
     */
    private LoggerFactory()
    {
        throw new AssertionError();
    }

    /**
     * Retrieves the {@link Logger} with the same name as the fully qualified
     * name of the {@link Class} provided from the current {@link LoggerContext}.
     * This method will always return a valid {@link Logger}. If no {@link Logger}
     * named with the {@link Class}'s FQCN, then one will be created.
     * @param clazz the {@link Class}, the name of which will be used to retrieve
     * a {@link Logger}.
     * @return a {@link Logger} with the same name as the provided {@link Class}
     */
    public static Logger getLogger(Class clazz)
    {
        return LoggerContext.get().getLogger(clazz);
    }

    /**
     * Returns the {@link Logger} with the given name. If the {@link Logger} does
     * not exist, it will be created.
     * @param name the name of the {@link Logger} required
     * @return the {@link Logger} with the name provided
     */
    public static Logger getLogger(String name)
    {
        return LoggerContext.get().getLogger(name);
    }

    /**
     * Retrieves the root {@link Logger} of the current {@link LoggerContext}
     * @return the root {@link Logger} of the current {@link LoggerContext}
     * @see LoggerContext#getRootLogger()
     * @see LoggerContext#get()
     */
    public static Logger getRootLogger()
    {
        return LoggerContext.get().getRootLogger();
    }
}
