package net.rumati.logging.muffero.appender;

import net.rumati.logging.muffero.LogEvent;
import net.rumati.logging.muffero.Logger;
import net.rumati.logging.muffero.LoggerFactory;
import net.rumati.logging.muffero.Priority;

/**
 * An appender that sends {@link LogEvent}s to all the appenders of the {@link Logger}'s parent.
 * @author Jesse Long
 */
public class InheritingAppender
        implements Appender
{
    public boolean isPriorityEnabled(String loggerName, Priority priority)
    {
        for (Appender appender : LoggerFactory.getLogger(loggerName).getAppenders()){
            if (appender.isPriorityEnabled(loggerName, priority)){
                return true;
            }
        }
        
        return false;
    }

    public void append(LogEvent event)
    {
        for (Appender appender : LoggerFactory.getLogger(event.getLoggerName()).getAppenders()){
            appender.append(event);
        }
    }

    public void shutdown()
    {
        /*
         * Do nothing. We shut down parent appenders when they are removed from
         * the parent logger.
         */
    }
}
