package net.rumati.logging.muffero.config.prop;

import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.TreeMap;
import net.rumati.logging.muffero.appender.Appender;
import net.rumati.logging.muffero.config.Configurator;
import net.rumati.logging.muffero.Logger;
import net.rumati.logging.muffero.LoggerFactory;
import net.rumati.logging.muffero.LoggingException;

public class PropertyConfigurator
        extends Configurator
{
    private final List<PropertyObjectLoader> loaders = new LinkedList<PropertyObjectLoader>();
    private final Map<String, String> config = new TreeMap<String, String>();

    public PropertyConfigurator(Properties properties)
    {
        this(propertiesToMap(properties));
    }

    private static Map<String, String> propertiesToMap(Properties properties)
    {
        Map<String, String> config = new TreeMap<String, String>();
        for (Map.Entry<Object, Object> e : properties.entrySet()){
            Object key = e.getKey();
            Object value = e.getValue();

            if (key != null && value != null){
                config.put(key.toString(), value.toString());
            }
        }
        return config;
    }

    public PropertyConfigurator(Map<String, String> config)
    {
        loaders.add(new DefaultPropertyObjectLoader());
        for (Map.Entry<String, String> e : config.entrySet()){
            this.config.put(e.getKey().trim(), e.getValue().trim());
        }
        for (Map.Entry<String, String> e : this.config.entrySet()){
            for (Map.Entry<String, String> e2 : this.config.entrySet()){
                if (!e2.getKey().equals(e.getKey())){
                    e2.setValue(e2.getValue().replace("${" + e.getKey() + "}", e.getValue()));
                }
            }
        }
    }

    public void addObjectLoader(PropertyObjectLoader loader)
    {
        loaders.add(loader);
    }

    public void configure()
            throws LoggingException
    {
        String loaderList = config.get("muffero.loaders");
        while (loaderList != null){
            String loaderClassName;
            int idx = loaderList.indexOf(",");
            if (idx < 0){
                loaderClassName = loaderList.trim();
                loaderList = null;
            }else{
                loaderClassName = loaderList.substring(0, idx).trim();
                loaderList = loaderList.substring(idx + 1).trim();
            }

            try{
                Class c = Class.forName(loaderClassName);
                if (!PropertyObjectLoader.class.isAssignableFrom(c)){
                    throw new LoggingException("Class \"" + c.getName() + "\" does not implement class \"" + PropertyObjectLoader.class.getName() + "\"");
                }
                try{
                    PropertyObjectLoader l = (PropertyObjectLoader)c.newInstance();
                    loaders.add(l);
                }catch (IllegalAccessException e){
                    throw new LoggingException("Error instantiating object loader " + loaderClassName, e);
                }catch (InstantiationException e){
                    throw new LoggingException("Error instantiating object loader " + loaderClassName, e);
                }catch (ExceptionInInitializerError e){
                    throw new LoggingException("Error instantiating object loader " + loaderClassName, e);
                }
            }catch (ClassNotFoundException e){
                throw new LoggingException("Unable to load object loader class: " + loaderClassName, e);
            }
        }

        PropertyConfigContext ctx = new PropertyConfigContext(loaders, config);

        for (Map.Entry<String, String> configEntry : config.entrySet()){
            String prefix = "muffero.logger.";
            String key = configEntry.getKey();
            if (key.startsWith(prefix) && key.length() > prefix.length()){
                String loggerName = key.substring(prefix.length());
                loadLoggerConfig(LoggerFactory.getLogger(loggerName), configEntry.getValue(), ctx);
            }
        }

        String rootLoggerConfig = config.get("muffero.rootLogger");
        if (rootLoggerConfig != null){
            loadLoggerConfig(LoggerFactory.getRootLogger(), rootLoggerConfig, ctx);
        }
    }

    private void loadLoggerConfig(Logger logger, String configuration, PropertyConfigContext ctx)
            throws LoggingException
    {
        String[] parts = configuration.split(",");

        for (String part : parts){
            part = part.trim();

            if (!part.isEmpty()){
                if (part.equals("NOINHERIT")){
                    logger.setInheritParentAppenders(false);
                }else{
                    Appender appender = ctx.loadAppender(part, "INVALID", config);
                    addAppender(logger, appender);
                }
            }
        }
    }
}
