package net.rumati.logging.muffero.config.prop;

import java.util.Map;
import java.util.Properties;
import net.rumati.logging.muffero.LoggingException;
import net.rumati.logging.muffero.appender.Appender;
import net.rumati.logging.muffero.layout.Layout;

/**
 * Loads objects from properties configuration. The {@link PropertyConfigurator}
 * configures Muffero using {@link Properties}. Each object (like
 * {@link Appender}s, {@link Layout}s etc) is configured in the properties.
 * A {@link PropertyObjectLoader} uses these properties to instantiate the
 * objects required to configure the system.
 * @author Jesse Long
 */
public interface PropertyObjectLoader
{
    /**
     * Indicates if this {@link PropertyObjectLoader} can load the required
     * object.
     * @param className The name of the class required to be instantiated.
     * @param c The generic interface type this class represents. (eg. {@link Layout}).
     * @return a {@code boolean} indicating if this {@link PropertyObjectLoader} can
     * be loaded by this loader.
     */
    public boolean canLoad(String className, Class<?> c);

    /**
     * Loads an object from the property configuration.
     * @param <T> The type of the object required. This will be an interface like {@link Layout}.
     * @param className The name of the class required to be loaded.
     * @param c The {@link Class} of the type required.
     * @param configuration The configuration of this object. Any prefixes will be stripped
     * @param ctx The {@link PropertyConfigContext} which can be used to load dependencies.
     * @return The required object
     * @throws LoggingException If the object could not be loaded.
     */
    public <T> T loadObject(String className, Class<T> c, Map<String, String> configuration, PropertyConfigContext ctx)
        throws LoggingException;
}
